@@ -50,17 +50,18 @@ class MLDL:
             return pd.DataFrame(self.queue.get())
 
         # MISS
-        maxi = self.cache[stage.upper()][max_length]['maxi']
-
         if self.filler_thread_cache is not None:
             self.kill_event_cache.set()
             self.filler_thread_cache.join()
             self.kill_event_cache.clear()
             self.filler_thread_cache = None
+            self.sample_cache = []
+
+        maxi = self.cache[stage.upper()][max_length]['maxi']
 
         if min_num_feature > 11:
-            c = col.find({'len': {'$lt': max_length + 1}, 'token_ids': {'$size': {'$gt': min_num_feature - 1}}},
-                         {'row_id': 1, '_id': 0}).sort("rand").limit(5 * num)
+            c = col.find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}},
+                         {'row_id': 1, '_id': 0}).sort("rand").limit(10 * num)
             self.sample_cache = list(map(lambda x: x['row_id'], list(c)))
             self.filler_thread_cache = Thread(target=self.background_sample_cache,
                                               args=(max_length, num, min_num_feature, col))
@@ -94,9 +95,9 @@ class MLDL:
                 break
 
     def background_sample_cache(self, max_length, num, min_num_feature, col):
-        i = 5
+        i = 10
         while i > 0:
-            c = col.find({'len': {'$lt': max_length + 1}, 'token_ids': {'$size': {'$gt': min_num_feature - 1}}},
+            c = col.find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}},
                          {'row_id': 1, '_id': 0}).sort("rand").skip(i * num).limit(num)
             tmp = list(c)
             self.sample_cache += list(map(lambda x: x['row_id'], tmp))
@@ -106,11 +107,16 @@ class MLDL:
                 break
 
     def terminate(self):
-        self.kill_event.set()
-        self.queue.get()
-        self.filler_thread.join()
-        self.kill_event.clear()
-        self.kill_event_cache.set()
-        self.filler_thread_cache.join()
-        self.kill_event_cache.clear()
-        self.filler_thread_cache = None
\ No newline at end of file
+        if self.filler_thread is not None:
+            self.kill_event.set()
+            self.queue.get()
+            self.filler_thread.join()
+            self.kill_event.clear()
+            self.queue = Queue(self.max_queue_size)
+
+        if self.filler_thread_cache is not None:
+            self.kill_event_cache.set()
+            self.filler_thread_cache.join()
+            self.kill_event_cache.clear()
+            self.filler_thread_cache = None
+            self.sample_cache = []
